// Copyright 2016 Orus.io. All rights reserved.

package cmd

import (
	"context"
	"fmt"
	"os"

	"github.com/olekukonko/tablewriter"
	"github.com/spf13/cobra"

	"xbus.io/go-xbus/api"
)

var pmCmd = &cobra.Command{
	Use:   "pm",
	Short: "Postmortem process management",
	Run: func(cmd *cobra.Command, args []string) {
		c, err := GetClient(false)
		if err != nil {
			Logger.Errorf("%s", err)
			os.Exit(1)
		}
		ctx, _ := contextCancelOnSignal(
			context.Background(), os.Interrupt,
		)
		list, err := c.PMProcessQuery(ctx, api.LogLevel_WARNING, false, []api.UUID{}, false)
		if err != nil {
			Logger.Errorf("%s", err)
			os.Exit(1)
		}
		table := tablewriter.NewWriter(os.Stdout)
		table.SetHeader([]string{"id", "level", "status", "comment"})
		for _, p := range list {
			table.Append([]string{
				p.Process.ID.String(),
				p.Level.String(),
				p.Status.String(),
				p.Comment,
			})
		}
		table.Render()
	},
}

var pmResetCmd = &cobra.Command{
	Use:   "reset",
	Short: "Reset a postportem process status to 'new'",
	Run: func(cmd *cobra.Command, args []string) {
		os.Exit(setStatus(
			api.MustUUIDFromString(args[0]),
			api.PMProcess_NEW,
			args[1]))
	},
}

var pmOpenCmd = &cobra.Command{
	Use:   "open",
	Short: "Indicate a postportem process is being addressed",
	Run: func(cmd *cobra.Command, args []string) {
		os.Exit(setStatus(
			api.MustUUIDFromString(args[0]),
			api.PMProcess_OPENED,
			args[1]))
	},
}

var pmCloseCmd = &cobra.Command{
	Use:   "close",
	Short: "Indicate a postportem process is closed",
	Run: func(cmd *cobra.Command, args []string) {
		fmt.Println(args)
		os.Exit(setStatus(
			api.MustUUIDFromString(args[0]),
			api.PMProcess_CLOSED,
			args[1]))
	},
}

func setStatus(processID api.UUID, status api.PMProcess_Status, comment string) int {
	c, err := GetClient(false)
	if err != nil {
		Logger.Errorf("%s", err)
		return 1
	}
	if err := c.PMProcessSetStatus(processID, status, comment); err != nil {
		Logger.Errorf("%s", err)
		return 1
	}
	return 0
}

func init() {
	RootCmd.AddCommand(pmCmd)
	pmCmd.AddCommand(pmResetCmd)
	pmCmd.AddCommand(pmOpenCmd)
	pmCmd.AddCommand(pmCloseCmd)
}
