package fullenv

import (
	"time"

	xbus "xbus.io/go-xbus"
	"xbus.io/go-xbus/api"
)

// NewEmitter initializes a Emitter
func NewEmitter(logger xbus.Logger) *Emitter {
	return &Emitter{
		log: logger,
	}
}

// Emitter is an emitter actor for FullEnv
type Emitter struct {
	Actor *xbus.Actor
	log   xbus.Logger
}

// Service is a Actor factory
func (e *Emitter) Service(actor *xbus.Actor) xbus.ActorService {
	e.Actor = actor
	return e
}

// Startup ...
func (e *Emitter) Startup() error {
	return nil
}

// Shutdown ...
func (e *Emitter) Shutdown() error {
	return nil
}

// NewConsumer initializes a Consumer
func NewConsumer(logger xbus.Logger) *Consumer {
	return &Consumer{
		log: logger,
	}
}

// Consumer is an emitter actor for FullEnv
type Consumer struct {
	Actor *xbus.Actor
	log   xbus.Logger

	Envelopes []*api.Envelope
}

// Service is a Actor factory
func (c *Consumer) Service(actor *xbus.Actor) xbus.ActorService {
	c.Actor = actor
	return c
}

// Process is called by xbus when inputs are ready
func (c *Consumer) Process(apc *xbus.ActorProcessingContext) error {
	env, err := apc.CompleteEnvelope(api.DefaultInput, time.Second)
	if err != nil {
		c.log.Errorf("Error receiving envelope: %s", err)
		return err
	}
	c.Envelopes = append(c.Envelopes, env)
	if c.Actor.GetIntSettingD("autosuccess", 0) != 1 {
		apc.Detach()
	}
	return nil
}
