package fullenv

import (
	"os"
	"path"
	"testing"

	"github.com/stretchr/testify/assert"
	yaml "gopkg.in/yaml.v2"
)

func TestApplyVars(t *testing.T) {
	for _, tt := range []struct {
		name     string
		initial  string
		vars     map[string]string
		expected string
	}{
		{"complex",
			`key: $V1
sub:
  key: $V2
list:
- key: $V3`,
			map[string]string{"V1": "value1", "V2": "value2", "V3": "value3"},
			`key: value1
sub:
  key: value2
list:
- key: value3
`,
		},
	} {
		t.Run(tt.name, func(t *testing.T) {
			var v RawSettings
			if err := yaml.Unmarshal([]byte(tt.initial), &v); err != nil {
				t.Fatal(err)
			}
			out, err := applyVars(v, tt.vars)
			if err != nil {
				t.Fatal(err)
			}
			b, err := yaml.Marshal(&out)
			if err != nil {
				t.Fatal(err)
			}
			assert.Equal(t, tt.expected, string(b))
		})
	}
}

func TestActorConfig(t *testing.T) {
	env, _ := NewFullEnvForTest(t, "actor-config", "fullenv-actor-config.yaml")
	defer env.Close()

	if err := env.Init(); err != nil {
		t.Fatal(err)
	}

	line, err := env.XbusCtl.RunReadLastLine("actor", "config", "main")
	if err != nil {
		t.Fatal(err)
	}
	assert.Equal(t, "http.webhook\t= http://example.local/ahook?key=plouf", line)
}

func TestCustomCtl(t *testing.T) {
	env, _ := NewFullEnvForTest(t, "actor-config", "fullenv-custom-ctl.yaml")
	defer env.Close()

	if err := env.Init(); err != nil {
		t.Fatal(err)
	}

	for _, f := range []string{"ctl/xbusctl.yaml",
		"ctl/ctl.certs",
		"ctl-ctl2/xbusctl.yaml",
		"ctl-ctl2/ctl2.certs",
	} {
		_, err := os.Stat(path.Join(env.dirname, f))
		assert.Nil(t, err)
	}
}
