package fullenv

import (
	"errors"
	"regexp"
	"testing"
	"time"

	xbus "xbus.io/go-xbus"
	"xbus.io/go-xbus/api"
	"github.com/stretchr/testify/assert"
)

func newBasRelayActor(actor *xbus.Actor) xbus.ActorService {
	return &badRelayActor{actor}
}

type badRelayActor struct {
	actor *xbus.Actor
}

func (a *badRelayActor) Process(apc *xbus.ActorProcessingContext) error {
	if _, err := apc.CompleteEnvelope(api.DefaultInput, time.Second); err != nil {
		apc.Log.Errorf("Reception error: %s", err)
		return err
	}
	return errors.New("badRelayActor error")
}

func TestInvalidProcessState(t *testing.T) {
	fe, _ := NewFullEnvForTest(t, "invalid-process-state", "invalid_process_state.yaml")

	if err := fe.Init(); err != nil {
		t.Fatal(err)
	}
	defer fe.Close()

	fe.RegisterActorService("badrelayactor", newBasRelayActor)

	fe.Startup()

	// Emit one message
	sender := fe.Emitter.Actor.OpenOutput(nil, "default", "msg")
	if err := sender.AddItems("msg", api.Item("hi")); err != nil {
		t.Fatal(err)
	}
	if err := sender.Close(); err != nil {
		t.Fatal(err)
	}

	// Wait for the process to be over
	time.Sleep(100 * time.Millisecond)

	// EXTRACT PIDS)
	re, err := regexp.Compile("\\| ([0-9a-f-]+) \\| [^\\|]+ \\| [^\\|]+ \\| [^\\|]+ \\| [A-Z]+ +\\|")

	// The relay will misbehave, and the process should pass in a 'error' state
	matchLines, err := fe.XbusCtl.RunReadAndFindSubmatch(re, "ps", "--status-error")
	must(err)
	assert.Equal(t, 1, len(matchLines), "Should get 1 matching line, got:\n%v", matchLines)

	// Restarting xbus should not be an issue
}
