package services

import (
	"fmt"

	"xbus.io/go-xbus"
)

// NewManager initialize a Manager
func NewManager(logger xbus.Logger) *Manager {
	return &Manager{
		Log: logger,
	}
}

// Manager ...
type Manager struct {
	services []Service
	running  []Service

	Log xbus.Logger
}

// Add adds a service
func (s *Manager) Add(service Service) {
	s.services = append(s.services, service)
}

// Startup starts all the services
func (s *Manager) Startup() error {
	for _, service := range s.services {
		if err := service.Startup(); err != nil {
			return fmt.Errorf("Could not start service %s: %s", service.GetName(), err)
		}
		s.running = append([]Service{service}, s.running...)
	}
	return nil
}

// Shutdown stops running services
func (s *Manager) Shutdown() error {
	var lastErr error
	for _, service := range s.running {
		if err := service.Shutdown(); err != nil {
			s.Log.Errorf("Error shutting down service %s: %s", service.GetName(), err)
			lastErr = err
		}
	}
	s.running = nil
	return lastErr
}
