package storage

import (
	"errors"

	"xbus.io/go-xbus/api"
)

var (
	// ErrInvalidPipelineStatus is returned when the pipeline status does not
	// allow the operation
	ErrInvalidPipelineStatus = errors.New("Invalid Pipeline Status")
)

// Pipeline defines how an envelope should be processed
type Pipeline struct {
	api.PipelineInfo
	Graph Graph
}

// PipelineStorage handle Pipeline persistence
type PipelineStorage interface {
	Load(api.UUID) (Pipeline, bool)
	Save(Pipeline) (api.UUID, error)
	SetStatus(api.UUID, api.PipelineInfo_Status) error
	LoadActivePipeline() []Pipeline
	Query(name, version string, activeOnly bool) []Pipeline
}
