package model

import (
	"fmt"

	"github.com/slicebit/qb"

	"xbus.io/go-xbus"
)

// QBLogger implements qb.Logger on top of a xbus.Logger
type QBLogger struct {
	logger   xbus.Logger
	logFlags qb.LogFlags
}

// Print is equivalent to xbus.Logger.Noticef()
func (l QBLogger) Print(v ...interface{}) {
	l.logger.Debugf(fmt.Sprint(v...))
}

// Printf is equivalent to xbus.Logger.Noticef()
func (l QBLogger) Printf(f string, v ...interface{}) {
	l.logger.Debugf(f, v...)
}

// Println is equivalent to xbus.Logger.Noticef()
func (l QBLogger) Println(v ...interface{}) {
	l.logger.Debugf(fmt.Sprintln(v...))
}

// Fatal is equivalent to xbus.Logger.Fatalf()
func (l QBLogger) Fatal(v ...interface{}) {
	l.logger.Fatalf(fmt.Sprint(v...))
}

// Fatalf is equivalent to xbus.Logger.Fatalf()
func (l QBLogger) Fatalf(f string, v ...interface{}) {
	l.logger.Fatalf(f, v...)
}

// Fatalln is equivalent to xbus.Logger.Fatalf()
func (l QBLogger) Fatalln(v ...interface{}) {
	l.logger.Fatalf(fmt.Sprintln(v...))
}

// Panic is equivalent to xbus.Logger.Errorf() + panic
func (l QBLogger) Panic(v ...interface{}) {
	s := fmt.Sprint(v...)
	l.logger.Errorf(s)
	panic(s)
}

// Panicf is equivalent to xbus.Logger.Errorf() + panic
func (l QBLogger) Panicf(f string, v ...interface{}) {
	s := fmt.Sprintf(f, v...)
	l.logger.Errorf(s)
	panic(s)
}

// Panicln is equivalent to xbus.Logger.Errorf() + panic
func (l QBLogger) Panicln(v ...interface{}) {
	s := fmt.Sprintln(v...)
	l.logger.Errorf(s)
	panic(s)
}

// LogFlags returns the current log flags
func (l QBLogger) LogFlags() qb.LogFlags {
	return l.logFlags
}

// SetLogFlags changes the current log flags
func (l *QBLogger) SetLogFlags(flags qb.LogFlags) {
	l.logFlags = flags
}
