// Copyright 2016 Orus.io. All rights reserved.

package sql

import (
	"database/sql"

	"github.com/orus-io/yago"

	xbus "xbus.io/go-xbus"
)

// RawSQLBase provides a base type for implementing pure-sql storages
type RawSQLBase struct {
	db     *yago.DB
	logger xbus.Logger
}

// MustBegin begins a transaction or panics
func (sb *RawSQLBase) MustBegin() *RawSQLTxHelper {
	return RawSQLMustBegin(sb.db, sb.logger)
}

// RawSQLTxHelper wraps a SQL tx and make it easier to enforce commit or rollback
type RawSQLTxHelper struct {
	*sql.Tx
	complete bool
	logger   xbus.Logger
}

// RawSQLMustBegin starts a sql transaction and wrap it
func RawSQLMustBegin(db *yago.DB, logger xbus.Logger) *RawSQLTxHelper {
	tx, err := db.Engine.DB().Begin()
	if err != nil {
		logger.Errorf("Cannot start a transaction: %s", err)
		panic(err)
	}
	return &RawSQLTxHelper{tx, false, logger}
}

// Commit commits the transaction
func (tx *RawSQLTxHelper) Commit() error {
	err := tx.Tx.Commit()
	tx.complete = true
	if err != nil {
		tx.logger.Errorf("Error on commit: %s", err)
	}
	return err
}

// MustCommit commits of rollback
func (tx *RawSQLTxHelper) MustCommit() {
	err := tx.Commit()
	if err != nil {
		panic(err)
	}
}

// Rollback rollbacks and log error if any
func (tx *RawSQLTxHelper) Rollback() error {
	err := tx.Tx.Rollback()
	tx.complete = true
	if err != nil {
		tx.logger.Errorf("Error on rollback: %s", err)
	}
	return err
}

// RollbackIfOpened rollback the transaction is still open
func (tx *RawSQLTxHelper) RollbackIfOpened() {
	if !tx.complete {
		tx.Rollback()
	}
}
