package testutils

import (
	"math/rand"
	"net"
	"strconv"
	"time"
)

// PortLock is a locker which locks by binding to a port on the loopback IPv4 interface
type PortLock struct {
	hostport string
	ln       net.Listener
}

// NewPortLock returns a lock for port
func NewPortLock(port int) *PortLock {
	return &PortLock{hostport: net.JoinHostPort("127.0.0.1", strconv.Itoa(port))}
}

// Lock locks on port
func (p *PortLock) Lock() error {
	eb := &expBackoff{time.Second}
	for {
		if ok, err := p.TryLock(); ok {
			return err
		}
		eb.Sleep()
	}
}

// TryLock acquires the lock, non-blocking
func (p *PortLock) TryLock() (bool, error) {
	if l, err := net.Listen("tcp", p.hostport); err == nil {
		p.ln = l // thanks to zhangpy
		return true, nil
	}
	return false, nil
}

// Unlock unlocks the port lock
func (p *PortLock) Unlock() error {
	if p.ln == nil {
		return nil
	}
	err := p.ln.Close()
	p.ln = nil
	return err
}

type expBackoff struct {
	time.Duration
}

func (eb *expBackoff) Sleep() {
	time.Sleep(eb.Duration)
	// next sleep length will be in [t, 2t)
	eb.Duration += time.Duration(float32(eb.Duration) * rand.Float32())
}
