package mocks

import (
	"fmt"

	"xbus.io/go-xbus/api"
	"bitbucket.org/orus-io/xbus/storage"
)

// NewProcessStorage returns a new ProcessStorage
func NewProcessStorage() *ProcessStorage {
	return &ProcessStorage{
		processMap:    make(map[api.UUID]storage.Process),
		emitPositions: make(map[storage.Emission]storage.EnvelopeReadPos),
	}
}

// ProcessStorage implements storage.ProcessStorage
type ProcessStorage struct {
	processMap    map[api.UUID]storage.Process
	emitPositions map[storage.Emission]storage.EnvelopeReadPos
}

// GetProcess returns a process data
func (s *ProcessStorage) GetProcess(id api.UUID) (storage.Process, error) {
	p, ok := s.processMap[id]
	if !ok {
		return storage.Process{}, fmt.Errorf("No such process: %s", id)
	}
	return p, nil
}

// GetProcessByTrigger returns a process matching a trigger or an empty value if not found
func (s *ProcessStorage) GetProcessByTrigger(emitter api.Actor, envelopeID api.UUID) (storage.Process, error) {
	for _, p := range s.processMap {
		if p.TriggerEmitterID == emitter.ID && p.TriggerEnvelopeID == envelopeID {
			return p, nil
		}
	}
	return storage.Process{}, nil
}

// CreateProcess saves a process
func (s *ProcessStorage) CreateProcess(p storage.Process) (api.UUID, error) {
	id, err := api.UUIDV4()
	if err != nil {
		return api.UUID{}, err
	}
	p.ID = id
	s.processMap[p.ID] = p
	return p.ID, nil
}

// ListActiveEmissions returns the active emissions
func (s *ProcessStorage) ListActiveEmissions(targetActor api.UUID) ([]storage.Emission, error) {
	var list []storage.Emission
	for emission, pos := range s.emitPositions {
		if !pos.Complete {
			list = append(list, emission)
		}
	}
	return list, nil
}
