package storage

import (
	"testing"

	"xbus.io/go-xbus/api"
	"bitbucket.org/orus-io/xbus/storage"
)

const purgeBatchSize = 5000

type massCreateProcessFunc func(ps storage.ProcessStorage, count int, status api.Process_Status) []api.UUID

// ProcessStorageBenchmark is a benchmark suite for a ProcessStorage implementation
func ProcessStorageBenchmark(b *testing.B, factory ProcessStorageFactory, massCreateProcess massCreateProcessFunc) {
	if massCreateProcess == nil {
		massCreateProcess = func(ps storage.ProcessStorage, count int, status api.Process_Status) []api.UUID {
			var ids = make([]api.UUID, 0, count)
			for i := 0; i != count; i++ {
				id, err := ps.CreateProcess(storage.Process{})
				if err != nil {
					b.Fatal(err)
				}
				if err := ps.SetStatus(id, status, ""); err != nil {
					b.Fatal(err)
				}
				ids = append(ids, id)
			}
			return ids
		}
	}
	b.Run("Purge", func(b *testing.B) {
		ps, clean := factory(b)
		defer clean()
		var ids = massCreateProcess(ps, b.N, api.Process_DONE)

		b.ResetTimer()
		for i := 0; i < b.N/purgeBatchSize; i++ {
			if res, err := ps.PurgeProcess(ids[i*purgeBatchSize : i*purgeBatchSize+purgeBatchSize]); err != nil {
				b.Fatal(b)
			} else if len(res) != purgeBatchSize {
				b.Fatalf("Expected %d process purged, got: %d", purgeBatchSize, len(res))
			}
		}
		if res, err := ps.PurgeProcess(ids[b.N/purgeBatchSize*purgeBatchSize:]); err != nil {
			b.Fatal(b)
		} else if len(res) != b.N%purgeBatchSize {
			b.Fatalf("Expected %d process purged, got: %d", b.N%purgeBatchSize, len(res))
		}
	})
}
