// Copyright 2017 Orus.io. All rights reserved.

package xbus

import (
	"fmt"
	"time"

	api "xbus.io/go-xbus/api"
)

// ActorLogger provides a Actor-based logging api
type ActorLogger struct {
	log   Logger
	actor *Actor
}

func (al ActorLogger) addLogMessage(context *api.ProcessingContext,
	env *api.Envelope, level api.LogLevel, format string, fmtArgs ...interface{}) {
	text := fmt.Sprintf(format, fmtArgs...)
	switch level {
	case api.LogLevel_NOTICE, api.LogLevel_WARNING:
		al.log.Noticef(text)
	case api.LogLevel_ERROR:
		al.log.Errorf(text)
	}
	if err := al.actor.SendLogs(context,
		env,
		[]api.LogMessage{
			api.LogMessage{
				Time:  time.Now(),
				Level: level,
				Text:  text,
			},
		}); err != nil {
		al.log.Errorf("Could not send the logs to xbus. Got '%s'", err)
	}
}

// Noticef logs a notif message
func (al ActorLogger) Noticef(format string, fmtArgs ...interface{}) {
	al.addLogMessage(nil, nil, api.LogLevel_NOTICE, format, fmtArgs...)
}

// EnvNoticef logs a envelope-related notif message
func (al ActorLogger) EnvNoticef(
	context *api.ProcessingContext,
	env *api.Envelope, format string, fmtArgs ...interface{},
) {
	al.addLogMessage(context, env, api.LogLevel_NOTICE, format, fmtArgs...)
}

// Warningf logs a warning message
func (al ActorLogger) Warningf(format string, fmtArgs ...interface{}) {
	al.addLogMessage(nil, nil, api.LogLevel_WARNING, format, fmtArgs...)
}

// EnvWarningf logs a envelope-related warning message
func (al ActorLogger) EnvWarningf(
	context *api.ProcessingContext,
	env *api.Envelope, format string, fmtArgs ...interface{},
) {
	al.addLogMessage(context, env, api.LogLevel_WARNING, format, fmtArgs...)
}

// Errorf logs a error message
func (al ActorLogger) Errorf(format string, fmtArgs ...interface{}) {
	al.addLogMessage(nil, nil, api.LogLevel_ERROR, format, fmtArgs...)
}

// EnvErrorf logs a envelope-related error message
func (al ActorLogger) EnvErrorf(
	context *api.ProcessingContext,
	env *api.Envelope, format string, fmtArgs ...interface{},
) {
	al.addLogMessage(context, env, api.LogLevel_ERROR, format, fmtArgs...)
}

// SendLogs sends log messages to the bus
func (a *Actor) SendLogs(
	context *api.ProcessingContext,
	envelope *api.Envelope,
	messages []api.LogMessage,
) error {
	var req = api.ActorLogRequest{Messages: messages,
		Context: context,
	}
	if envelope != nil {
		req.EnvelopeID = &envelope.ID
	}

	return a.Client.API.Director.Log(a.ID.String(), req)
}
