package cmd

import (
	"encoding/json"
	"io"
	"os"

	"github.com/spf13/cobra"

	"xbus.io/go-xbus"
	"xbus.io/go-xbus/api"
)

var emitCmd = cobra.Command{
	Use:   "emit",
	Short: "Emit arbitrary data with an emitter",
	Run: func(cmd *cobra.Command, args []string) {
		if len(args) == 0 || len(args) > 2 {
			cmd.Usage()
			os.Exit(1)
		}
		emitter := args[0]
		input := "-"
		if len(args) > 1 {
			input = args[1]
		}
		os.Exit(doEmit(emitter, input))
	},
}

func loadEnvelope(input string) (*api.Envelope, error) {
	var in io.Reader
	if input == "-" {
		in = os.Stdin
	} else {
		panic("soon")
	}
	decoder := json.NewDecoder(in)
	var envelope api.Envelope
	if err := decoder.Decode(&envelope); err != nil {
		return nil, err
	}
	return &envelope, nil
}

func eventTypes(envelope *api.Envelope) (types []string) {
	for _, event := range envelope.Events {
		types = append(types, event.Type)
	}
	return
}

func doEmit(emitter, input string) int {
	client, err := GetClient(false)
	if err != nil {
		Logger.Errorf("Error initializing: %s", err)
		return 1
	}

	if err := client.Startup(false); err != nil {
		Logger.Errorf("Error starting client: %s", err)
		return 1
	}
	defer client.Shutdown()

	var actor *xbus.Actor
	if actorID, err := api.UUIDFromString(emitter); err == nil {
		actor = client.GetActor(actorID)
	} else {
		actor = client.GetActorByName(emitter)
	}
	if actor == nil {
		Logger.Errorf("No such emitter: %s", err)
		return 1
	}

	envelope, err := loadEnvelope(input)

	sender := actor.OpenOutput(nil, "default", eventTypes(envelope)...)

	for _, event := range envelope.Events {
		if err := sender.AddItems(event.Type, event.Items...); err != nil {
			Logger.Errorf("Error adding items: %s", err)
			return 1
		}
	}

	if err := sender.Close(); err != nil {
		Logger.Errorf("Error sending: %s", err)
		return 1
	}

	return 0
}

func init() {
	RootCmd.AddCommand(&emitCmd)
}
