package cmd

import (
	"os"
	"os/signal"
	"syscall"

	"github.com/spf13/cobra"
)

var serveCmd = &cobra.Command{
	Use:   "serve",
	Short: "Start & serve the configured actors",
	Run: func(cmd *cobra.Command, args []string) {
		os.Exit(doServe())
	},
}

func doServe() int {
	client, err := GetClient(false)
	if err != nil {
		Logger.Errorf("Error initializing: %s", err)
		return 1
	}

	if err := client.Startup(true); err != nil {
		Logger.Errorf("Error starting: %s", err)
		return 1
	}

	Logger.Noticef("Now Running")
	// TODO do something a little more elaborate so that interrupt during startup
	// is gracefully handlded
	signals := make(chan os.Signal, 1)
	signal.Notify(signals, os.Interrupt, syscall.SIGTERM)

	select {
	case sig := <-signals:
		Logger.Noticef("Signal '%s' caught, shutting down", sig)
		client.Shutdown()
	case <-client.Done():
		Logger.Noticef("The client stopped by itself")
		return 1
	}

	return 0
}

func init() {
	RootCmd.AddCommand(serveCmd)
}
