package demo

import (
	"fmt"

	xbus "xbus.io/go-xbus"
	"xbus.io/go-xbus/envelope"
)

// NewHelloWorker is a HelloWorker factory
func NewHelloWorker(actor *xbus.Actor) xbus.ActorService {
	return &HelloWorker{}
}

// HelloWorker is a worker that forward incoming envelopes to the default output
// The service parameters are:
type HelloWorker struct{}

// Process handles an incoming process
func (w *HelloWorker) Process(apc *xbus.ActorProcessingContext) error {
	env, err := apc.GetEnvelope("default")
	if err != nil {
		return err
	}

	out := envelope.MustNewOpenedEnvelope()
	errC := make(chan error)
	go func() {
		errC <- apc.Send("default", out)
	}()

	if err := envelope.ForEachMessage(env, func(msg envelope.Message) error {
		outMsg, chunks := envelope.MustNewMessageChunkWriter(msg.MsgType())
		if err := out.Add(outMsg); err != nil {
			return err
		}
		if err := envelope.ForEachChunkString(msg, func(data string) error {
			return chunks.WriteString(
				fmt.Sprintf("Hello %s", data),
			)
		}); err != nil {
			return err
		}
		return chunks.Close()
	}); err != nil {
		out.CloseWithError(err)
		return err
	}

	if err := out.Close(); err != nil {
		return err
	}

	if err := <-errC; err != nil {
		return fmt.Errorf("could not send the envelope: %s", err)
	}
	return nil
}

func init() {
	xbus.RegisterActorService("demo.hello", NewHelloWorker)
}
