package demo

import (
	"sync"

	xbus "xbus.io/go-xbus"
	"xbus.io/go-xbus/envelope"
)

// NewPrintToConsoleConsumer is a PrintToConsoleConsumer factory
func NewPrintToConsoleConsumer(*xbus.Actor) xbus.ActorService {
	return &PrintToConsoleConsumer{}
}

// PrintToConsoleConsumer is a consumer that prints the received items on
// the console
type PrintToConsoleConsumer struct{}

// Process handles an incoming process
func (c *PrintToConsoleConsumer) Process(apc *xbus.ActorProcessingContext) error {
	var wg sync.WaitGroup

	for _, input := range apc.Request.Inputs {
		env, err := apc.GetEnvelope(input.Name)
		if err != nil {
			return err
		}

		if err := envelope.ForEachMessage(env, func(msg envelope.Message) error {
			wg.Add(1)
			go func(msg envelope.Message) {
				defer wg.Done()
				envelope.ForEachChunkString(msg, func(chunk string) error {
					apc.LocalLog.Noticef(
						"print-to-console - %s: %s", msg.MsgType(), chunk)
					return nil
				})
			}(msg)
			return nil
		}); err != nil {
			return err
		}
	}
	wg.Wait()
	return nil
}

func init() {
	xbus.RegisterActorService("demo.print-to-console", NewPrintToConsoleConsumer)
}
