package xbus

import (
	"errors"
	"time"

	api "xbus.io/go-xbus/api"
	nrpc "github.com/nats-rpc/gogo-nrpc"
)

func (c *Client) sendEnvelopeFragment(
	actorID api.UUID, ctx *api.ProcessingContext,
	output string, env *api.Envelope,
) (ack api.EnvelopeAck, err error) {
	delay := 50 * time.Millisecond
	broker := c.API.NewBroker(actorID.String())

	for {
		ack, err = broker.Output(api.OutputRequest{
			Context:  ctx,
			Output:   output,
			Envelope: env,
			Close:    env == nil,
		})
		if nrpcErr, _ := err.(*nrpc.Error); nrpcErr.GetType() == nrpc.Error_SERVERTOOBUSY {
			// wait a little
			time.Sleep(delay)
			if delay < time.Second*10 {
				delay = delay * 3 / 2
			}
			// try again
			continue
		}
		if err == nil {
			if ack.GetStatus() == api.EnvelopeAck_ERROR {
				err = errors.New(ack.GetReason())
			}
		} else {
			c.Log.Errorf("Error sending envelope: %s", err)
		}
		return
	}
}

// SendEnvelopeFragment sends an envelope on a Actor output
func (c *Client) SendEnvelopeFragment(
	actorID api.UUID, ctx *api.ProcessingContext,
	output string, env api.Envelope,
) (api.EnvelopeAck_ReceptionStatus, error) {
	ack, err := c.sendEnvelopeFragment(actorID, ctx, output, &env)
	return ack.GetStatus(), err
}
